// -*-C++-*-
// debug.cpp
//
//  The WorldForge Project
//  Copyright (C) 2000  The WorldForge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about WorldForge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//

#include <stdio.h>
#include "debug.h"

void CoalDebug::Dump (const CoalCoord &coord)
{
    printf ("Coord:   x = %4.2f, y = %4.2f, z = %4.2f\n",
	    coord.GetX (), coord.GetY (), coord.GetZ ());
}

void CoalDebug::Dump (CoalRect &rect)
{
    printf ("Rect:   top = %4.2f, left = %4.2f, bottom = %4.2f, right = %4.2f\n",
	    rect.GetTop(), rect.GetLeft(), rect.GetBottom(), rect.GetRight());
}

void CoalDebug::Dump (const CoalColor &color)
{
    printf ("Color: red = %d, green = %d, blue = %d\n",
            color.red, color.green, color.blue);
}

void CoalDebug::Dump (const CoalGraphic &graphic)
{
    printf ("Graphic [%ld] %s '%s'\n\t", graphic.id, graphic.name.c_str (),
            graphic.desc.c_str ());
    Dump (graphic.bkgdcolor);
    printf ("\tfilename: %s\n", graphic.filename.c_str ());
    printf ("\toffset: ");
    Dump (graphic.offset);
}

void CoalDebug::Dump (const CoalPolyGraphic &graphic)
{
    printf ("Poly");
    Dump (*(CoalGraphic*)(&graphic));  // Use CoalGraphic flavor first
    for (int i = 0; i < 8; i++)
    {
        if (!graphic.altfiles[i].empty ())
            printf ("\talt[%d]: %s\n", i, graphic.altfiles[i].c_str ());
    }
}

void CoalDebug::Dump (CoalFill &fill)
{
    printf ("Fill: %s\n\t", fill.name.c_str ());

    if (fill.graphic)
        Dump (*(CoalGraphic*)fill.graphic);
}

void CoalDebug::Dump (CoalShape &shape)
{
    printf ("\tFound %d Coords:\n", shape.GetCoordCount ());
    for (unsigned int i = 0; i < shape.GetCoordCount (); i++)
    {
        printf ("\t\t");
        Dump (shape.GetCoord (i));
    }
}

void CoalDebug::Dump (CoalObject &object)
{
    printf ("Object: %s\n", object.name.c_str ());
    printf ("\tdesc =  %s\n", object.desc.c_str ());
    Dump (object.anchor);  // Use CoalShape flavor
}

void CoalDebug::Dump (CoalRegion &region)
{
    printf ("Region: %s\n", region.name.c_str ());
    printf ("\ttype = %d\n\tlayer = %d\n", region.GetType (),
            region.GetLayer ());

    Dump (*(CoalShape*)(&region));  // Use CoalShape flavor

    CoalFill *fill = region.GetFill ();
    if (fill)
        Dump (*fill);
}

void CoalDebug::Dump (const CoalTile &tile)
{
    Dump (tile.graphic);
    Dump (tile.coord);
}

void CoalDebug::Dump (CoalQuery &query)
{
    QueryParameter param;
    query.GetQueryParameters (param);

    printf ("Query:   start = %d, end = %d\n",
	    param.GetStart (), param.GetEnd ());
}

void CoalDebug::Dump (CoalDatabase &database)
{
    int count, i;
    CoalPolyGraphic *graphic;
    CoalFill *fill;
    CoalRegion *region;
    CoalObject *object;

    count = database.GetGraphicCount ();
    printf ("Dumping %d graphics...\n", count);
    for (i = 0; i < count; i++)
    {
        graphic = (CoalPolyGraphic*)database.GetGraphic (i);
        if (graphic)
        {
            Dump (*graphic);
        }
    }

    count = database.GetFillCount ();
    printf ("Dumping %d fills...\n", count);
    for (i = 0; i < count; i++)
    {
        fill = (CoalFill*)database.GetFill (i);
        if (fill)
        {
            Dump (*fill);
        }
    }

    count = database.GetRegionCount ();
    printf ("\nDumping %d regions...\n", count);
    for (i = 0; i < count; i++)
    {
        region = (CoalRegion*)database.GetRegion (i);
        if (region)
        {
            Dump (*region);
            printf ("\n");
        }
    }

    count = database.GetObjectCount ();
    printf ("\nDumping %d objects...\n", count);
    for (i = 0; i < count; i++)
    {
        object = (CoalObject*)database.GetObject (i);
        if (object)
        {
            Dump (*object);
            printf ("\n");
        }
    }
    printf ("done dumping database\n");
}


#if 0
void CoalDebug::Dump (CoalMap &map, CoalDebugVerbosity verbose)
{
    CoalQuery query;
    query.SetExtents (1, 7);

    query.QueryAll ();
    Dump (query);
    if (!map.RunQuery (query))
	return;

    int items;
    CoalGraphic *gfx;

    items = query.data.GetCount ();
    for (int i = 0; i < items; i++)
    {
	gfx = (CoalGraphic*)query.data.GetItem (i);
	if (!gfx)
	    continue;

	Dump (*gfx);
    }
}
#endif

// Print out the shape as ASCII art.  Good for debugging
// IsPointIn() functions.
void CoalDebug::AsciiDump (CoalShape &shape)
{
    CoalCoord coord;
    CoalRect rect;
    shape.GetBoundingRect (rect);

    printf ("Dumping shape (type = %d)...\n", shape.GetType ());
    Dump (shape);

    int x, y;
    for (y = 0; y <= (int)rect.GetBottom() + 1; y++)
    {
        for (x = 0; x <= (int)rect.GetRight() + 1; x++)
        {
            coord.SetCoord (x, y);
            if (shape.IsPoint (coord))
                printf ("*");
            else if (shape.IsPointIn (coord))
                printf ("x");
            else
                printf (".");
        }
        printf ("\n");
    }
    printf ("\n");

}
