// -*-C++-*-
// mimloader.cpp
//
//  The WorldForge Project
//  Copyright (C) 2000  The WorldForge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about WorldForge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//

#include <stdio.h>

#include "mimloader.h"

bool CoalMimLoader::LoadMap (const char *mapfile)
{
    m_xmldoc = xmlParseFile (mapfile);
    if (!m_xmldoc)
        return false;

    ParseMIM ();
    return true;
}

bool CoalMimLoader::LoadMap (const char *mapbuffer, int buffersize)
{
    // Don't ask me why xmlParseMemory() doesn't use a const char*,
    // but xmlParseFile() does.  gnome-xml shouldn't have to change
    // the input buffer, should it?
    m_xmldoc = xmlParseMemory ((char*)mapbuffer, buffersize);
    return false;
}

bool CoalMimLoader::CleanupLoader ()
{
    xmlFreeDoc (m_xmldoc);

    CoalGraphicMapIter iter = m_graphics.begin ();
    while (iter != m_graphics.end())
    {
        printf ("...deleting graphic %s\n", (*iter).second->desc.c_str());
        delete ((*iter).second);
        iter++;
    }

    m_graphics.clear ();
    return true;
}

bool CoalMimLoader::IsElement (const xmlChar *nodeName, const char *name)
{
    return (!strcmp ((const char*)nodeName, name));
}

// Assign values in colortext into color
void CoalMimLoader::ParseColor (const string &colortext, CoalColor &color)
{
    int nread;
    nread = sscanf (colortext.c_str (), "%d,%d,%d",
                    &color.red, &color.green, &color.blue);

    //printf ("    COLOR: r: %d, g: %d, b: %d\n",
    //          color.red, color.green, color.blue);
}

void CoalMimLoader::Parse2DProperties (const string &proptext,
                                       CoalPolyGraphic *graphic)
{
    if (!graphic)
        return;

    unsigned int comma;
    comma = proptext.find (',');
    graphic->filename = proptext.substr (0, comma);
    Trim (graphic->filename);

    // FIXME: Doesn't handle altfiles[] yet...
}

// Maybe move this generic function to CoalMapLoader...?
int CoalMimLoader::Trim(string &str)
{
    if (str.size() == 0)
        return 0;

    unsigned int newstart = str.find_first_not_of (" \t\r\n");
    unsigned int newend = str.find_last_not_of (" \t\r\n");

    if (newend <= newstart)
    {
        str.erase ();
        return 0;
    }

    str = str.substr (newstart, newend + 1);

    return str.size ();  // New length
}

string CoalMimLoader::GetElementContents (const xmlNodePtr node)
{
    string contents = (const char*)xmlNodeGetContent (node);
    Trim (contents);
    return contents;
}

void CoalMimLoader::ParseMIM ()
{
    if (!m_xmldoc)
        return;

    // Parse through top-level elements
    xmlNodePtr nextNode = m_xmldoc->root->childs;
    while (nextNode)
    {
        if (IsElement (nextNode->name, "entity"))
            ParseMIMEntity (nextNode);
        else if (IsElement (nextNode->name, "region"))
            ParseMIMRegion (nextNode);
        else if (IsElement (nextNode->name, "polygon"))
            ParseMIMPolygon (nextNode);
        else if (IsElement (nextNode->name, "object"))
            ParseMIMObject (nextNode);

        nextNode = nextNode->next;
    }
}

void CoalMimLoader::ParseMIMEntity (xmlNodePtr entityNode)
{
    if (!entityNode || !m_db)
        return;

    // Always use CoalPolyGraphic for MIM maps
    CoalPolyGraphic *graphic = new CoalPolyGraphic;
    m_db->AddGraphic (graphic);

    // Parse through 2nd-level elements
    xmlNodePtr nextNode = entityNode->childs;
    while (nextNode)
    {
        if (IsElement (nextNode->name, "name"))
        {
            graphic->desc = (const char*)xmlNodeGetContent (nextNode);
            Trim (graphic->desc);
        }
        else if (IsElement (nextNode->name, "color"))
        {
            string colortext = (const char*)xmlNodeGetContent (nextNode);
            Trim (colortext);
            ParseColor (colortext, graphic->bkgdcolor);
        }
        else if (IsElement (nextNode->name, "prop_2d"))
        {
            // Put the first element into graphic->filename, and the
            // rest into graphic->altfiles[].
            string proptext = (const char*)xmlNodeGetContent (nextNode);
            Parse2DProperties (proptext, graphic);
        }
        else if (IsElement (nextNode->name, "type"))
        {
            // FIXME
            //printf ("  ENTITY %s: %s\n", nextNode->name, xmlNodeGetContent (nextNode));
        }
        else if (IsElement (nextNode->name, "prop_3d"))
        {
            // FIXME
            //printf ("  ENTITY %s: %s\n", nextNode->name, xmlNodeGetContent (nextNode));
        }

        nextNode = nextNode->next;
    }
}

void CoalMimLoader::ParseMIMRegion (xmlNodePtr regionNode)
{
    if (!regionNode)
        return;

    // Parse through 2nd-level elements
    xmlNodePtr nextNode = regionNode->childs;
    while (nextNode)
    {
        if (IsElement (nextNode->name, "name"))
        {
            printf ("  REGION %s: %s\n", nextNode->name, xmlNodeGetContent (nextNode));
        }
        else if (IsElement (nextNode->name, "seed"))
        {
            // FIXME
            //printf ("  REGION %s: %s\n", nextNode->name, xmlNodeGetContent (nextNode));
        }
        else if (IsElement (nextNode->name, "base_entity"))
        {
            // FIXME
            printf ("  REGION %s: %s\n", nextNode->name, xmlNodeGetContent (nextNode));
        }
        else if (IsElement (nextNode->name, "subtractive"))
        {
            // FIXME
            printf ("  REGION %s: %s\n", nextNode->name, xmlNodeGetContent (nextNode));
        }
        else if (IsElement (nextNode->name, "item"))
        {
            printf ("  REGION %s: %s\n", nextNode->name, xmlNodeGetContent (nextNode));
            xmlNodePtr nextItemNode = nextNode->childs;
            while (nextItemNode)
            {
                if (IsElement (nextItemNode->name, "entity_name"))
                {
                    // Use <entity_name> to find CoalGraphic to use
                    // with this item
                    printf ("    ITEM %s: %s\n", nextItemNode->name,
                            xmlNodeGetContent (nextItemNode));
                }
                else if (IsElement (nextItemNode->name, "frequency"))
                {
                    // FIXME
                    printf ("    ITEM %s: %s\n", nextItemNode->name,
                            xmlNodeGetContent (nextItemNode));
                }
                else if (IsElement (nextItemNode->name, "edging"))
                {
                    // FIXME
                    printf ("    ITEM %s: %s\n", nextItemNode->name,
                            xmlNodeGetContent (nextItemNode));
                }
                nextItemNode = nextItemNode->next;
            }
        }

        nextNode = nextNode->next;
    }
}

void CoalMimLoader::ParseMIMPolygon (xmlNodePtr polygonNode)
{
    if (!polygonNode)
        return;

    // Parse through 2nd-level elements
    xmlNodePtr nextNode = polygonNode->childs;
    while (nextNode)
    {
        if (IsElement (nextNode->name, "region_name"))
        {
            // FIXME
            printf ("  POLYGON %s: %s\n", nextNode->name, xmlNodeGetContent (nextNode));
        }
        else if (IsElement (nextNode->name, "layer"))
        {
            // FIXME
            printf ("  POLYGON %s: %s\n", nextNode->name, xmlNodeGetContent (nextNode));
        }
        else if (IsElement (nextNode->name, "vertices"))
        {
            // FIXME
            printf ("  POLYGON %s: %s\n", nextNode->name, xmlNodeGetContent (nextNode));
        }

        nextNode = nextNode->next;
    }
}

void CoalMimLoader::ParseMIMObject (xmlNodePtr objectNode)
{
    if (!objectNode)
        return;

    // Parse through 2nd-level elements
    xmlNodePtr nextNode = objectNode->childs;
    while (nextNode)
    {
        if (IsElement (nextNode->name, "object_name"))
        {
            // FIXME
            printf ("  OBJECT %s: %s\n", nextNode->name, xmlNodeGetContent (nextNode));
        }
        else if (IsElement (nextNode->name, "position"))
        {
            // FIXME
            printf ("  OBJECT %s: %s\n", nextNode->name, xmlNodeGetContent (nextNode));
        }

        nextNode = nextNode->next;
    }
}

